@@ -28,17 +28,16 @@
  */
 
 #include <sys/stat.h>
-#include <glade/glade.h>
 #include "pilot.h"
 #include "util.h"
 #include "gnome-pilot-cdialog.h"
 
 
-static GtkObjectClass *parent_class = NULL;
+static GObjectClass *parent_class = NULL;
 
 struct _GnomePilotCDialogPrivate 
 {
-	GladeXML *xml;
+	GtkBuilder *ui;
 
 	ConduitState *state;
 
@@ -60,28 +59,29 @@ static void fill_widgets (GnomePilotCDialog *gpcd);
 
 static void gpcd_action_activated (GtkWidget *widget, gpointer user_data);
 
-static void gpcd_destroy (GtkObject *object);
+static void gpcd_dispose (GObject *object);
 
-GtkType
+GType
 gnome_pilot_cdialog_get_type (void)
 {
-  static GtkType type = 0;
+  static GType type = 0;
 
   if (type == 0)
     {
-      static const GtkTypeInfo info =
+      static const GTypeInfo info =
       {
-        "GnomePilotCDialog",
-        sizeof (GnomePilotCDialog),
         sizeof (GnomePilotCDialogClass),
-        (GtkClassInitFunc) class_init,
-        (GtkObjectInitFunc) init,
-        /* reserved_1 */ NULL,
-        /* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL,
+        NULL,
+        NULL,
+        (GClassInitFunc) class_init,
+        NULL,
+        NULL,
+        sizeof (GnomePilotCDialog),
+	0,
+        (GInstanceInitFunc) init,
       };
 
-      type = gtk_type_unique (gtk_object_get_type (), &info);
+      type = g_type_register_static (g_object_get_type (), "GnomePilotCDialog", &info, 0);
     }
 
   return type;
@@ -90,33 +90,40 @@ gnome_pilot_cdialog_get_type (void)
 static void
 class_init (GnomePilotCDialogClass *klass)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 
-	object_class = GTK_OBJECT_CLASS (klass);
+	object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = gtk_type_class (gtk_object_get_type ());
+	parent_class = g_type_class_peek (g_object_get_type ());
 
-	object_class->destroy = gpcd_destroy;
+	object_class->dispose = gpcd_dispose;
 }
 
 static void
 init (GnomePilotCDialog *gpcd)
 {
 	GnomePilotCDialogPrivate *priv;
-	
+	guint error;
+	gchar *objects[] = {"ConduitSettings",
+			    NULL};
+
 	priv = g_new0 (GnomePilotCDialogPrivate, 1);
 
 	gpcd->priv = priv;
 
-	/* Gui stuff */
-	priv->xml = glade_xml_new (GLADEDATADIR "/gpilotd-capplet.glade", "ConduitSettings", NULL);
-	if (!priv->xml) {
-		g_message ("gnome-pilot-cdialog init(): Could not load the Glade XML file!");
-		goto error;
+	/* Gui stuff */ 
+	priv->ui = gtk_builder_new ();
+	error = gtk_builder_add_objects_from_file (priv->ui, "gpilotd-capplet.ui", objects, NULL);
+	if (error == 0) {
+		error = gtk_builder_add_objects_from_file (priv->ui, UIDATADIR "/gpilotd-capplet.ui", objects, NULL);
+		if (error == 0) {
+			g_message ("gnome-pilot-cdialog init(): Could not load the GtkBuilder UI file!");
+			goto error;
+		}
 	}
 
 	if (!get_widgets (gpcd)) {
-		g_message ("gnome-pilot-cdialog init(): Could not find all widgets in the XML file!");
+		g_message ("gnome-pilot-cdialog init(): Could not find all widgets in the UI file!");
 		goto error;
 	}
 	
@@ -126,13 +133,13 @@ init (GnomePilotCDialog *gpcd)
 
 
 
-GtkObject *
+GObject *
 gnome_pilot_cdialog_new (ConduitState *state)
 {
 	GnomePilotCDialog *gpcd;
-	GtkObject *object;
+	GObject *object;
 	
-	object = gtk_type_new (GNOME_PILOT_TYPE_CDIALOG);
+	object = G_OBJECT(g_type_create_instance(GNOME_PILOT_TYPE_CDIALOG));
 	
 	gpcd = GNOME_PILOT_CDIALOG (object);
 	gpcd->priv->state = state;
@@ -150,13 +157,13 @@ get_widgets (GnomePilotCDialog *gpcd)
 
 	priv = gpcd->priv;
 
-#define GW(name) glade_xml_get_widget (priv->xml, name)
+#define GW(name) GTK_WIDGET (gtk_builder_get_object (priv->ui, name))
 
 	priv->dialog = GW ("ConduitSettings");
 
 	priv->settings_frame = GW ("settings_frame");
-	priv->sync_actions = GW ("sync_actions_menu");
-	priv->sync_one_actions = GW ("sync_one_actions_menu");
+	priv->sync_actions = GW ("sync_actions_combo");
+	priv->sync_one_actions = GW ("sync_one_actions_combo");
 	
 	priv->options_frame = GW ("options_frame");
 
@@ -164,6 +171,7 @@ get_widgets (GnomePilotCDialog *gpcd)
 	return (priv->dialog
 		&& priv->settings_frame
 		&& priv->sync_actions
+		&& priv->sync_one_actions
 		&& priv->options_frame);
 }
 
@@ -171,21 +179,11 @@ static void
 init_widgets (GnomePilotCDialog *gpcd)
 {
 	GnomePilotCDialogPrivate *priv;
-	GtkWidget *menu;
-	GList *l;
 	
 	priv = gpcd->priv;
 
-	menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (priv->sync_actions));
-	l = GTK_MENU_SHELL (menu)->children;
-	while (l != NULL) {
-		GtkWidget *menu_item = GTK_WIDGET (l->data);
-		
-		gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
-				    GTK_SIGNAL_FUNC (gpcd_action_activated), gpcd);
-		
-		l = l->next;
-	}
+	g_signal_connect (G_OBJECT (priv->sync_actions), "changed",
+	    G_CALLBACK (gpcd_action_activated), gpcd);
 }
 
 static void
@@ -196,9 +194,8 @@ fill_widgets (GnomePilotCDialog *gpcd)
 	priv = gpcd->priv;
 
 	if (priv->state) {
-		fill_conduit_sync_type_menu (GTK_OPTION_MENU (priv->sync_actions), priv->state);
-		if (priv->state->default_sync_type != GnomePilotConduitSyncTypeCustom)
-			fill_conduit_first_sync_type_menu (GTK_OPTION_MENU (priv->sync_one_actions), priv->state);
+		fill_conduit_sync_type_combo (GTK_COMBO_BOX (priv->sync_actions), priv->state);
+		fill_conduit_first_sync_type_combo (GTK_COMBO_BOX (priv->sync_one_actions), priv->state);
 
 		if (!priv->state->has_settings) {
 			gtk_widget_hide (priv->settings_frame);
@@ -221,28 +218,36 @@ GnomePilotConduitSyncType
 gnome_pilot_cdialog_sync_type (GnomePilotCDialog *gpcd)
 {
 	GnomePilotCDialogPrivate *priv;
-	GtkWidget *menu, *menu_item;
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+	int sync_type = -1;
 	
 	priv = gpcd->priv;
 	
-	menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (priv->sync_actions));
-	menu_item = gtk_menu_get_active (GTK_MENU (menu));
-	
-	return GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (menu_item), "sync_type"));
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX(priv->sync_actions));
+	if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(priv->sync_actions), &iter)) {
+		gtk_tree_model_get(model, &iter, 1, &sync_type, -1);
+	}
+
+	return sync_type;
 }
 
 GnomePilotConduitSyncType 
 gnome_pilot_cdialog_first_sync_type (GnomePilotCDialog *gpcd)
 {
 	GnomePilotCDialogPrivate *priv;
-	GtkWidget *menu, *menu_item;
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+	int sync_type = -1;
 	
 	priv = gpcd->priv;
 	
-	menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (priv->sync_one_actions));
-	menu_item = gtk_menu_get_active (GTK_MENU (menu));
-	
-	return GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (menu_item), "sync_type"));
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX(priv->sync_one_actions));
+	if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(priv->sync_one_actions), &iter)) {
+		gtk_tree_model_get(model, &iter, 1, &sync_type, -1);
+	}
+
+	return sync_type;
 }
 
 gboolean
@@ -254,6 +259,7 @@ gnome_pilot_cdialog_run_and_close (GnomePilotCDialog *gpcd, GtkWindow *parent)
 	priv = gpcd->priv;
 	
 	gtk_window_set_transient_for (GTK_WINDOW (priv->dialog), parent);
+	fill_conduit_sync_type_combo (GTK_COMBO_BOX (priv->sync_actions), priv->state);
 	btn = gtk_dialog_run (GTK_DIALOG (priv->dialog));
 	gtk_widget_hide(priv->dialog);
 
@@ -276,15 +282,13 @@ gpcd_action_activated (GtkWidget *widget, gpointer user_data)
 }
 
 static void
-gpcd_destroy (GtkObject *object)
+gpcd_dispose (GObject *object)
 {
 	GnomePilotCDialog *gpcd = GNOME_PILOT_CDIALOG (object);
 	GnomePilotCDialogPrivate *priv;
 	
 	priv = gpcd->priv;
 
-	gtk_object_unref (GTK_OBJECT (priv->xml));
+	g_object_unref (G_OBJECT (priv->ui));
 
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
