@@ -25,10 +25,10 @@
 #ifndef GPILOTD_CAPPLET_PILOT_H
 #define GPILOTD_CAPPLET_PILOT_H
 
-#include <gpilotd/gnome-pilot-structures.h>
-#include <gpilotd/gnome-pilot-conduit.h>
-#include <libgpilotdCM/gnome-pilot-conduit-management.h>
-#include <libgpilotdCM/gnome-pilot-conduit-config.h>
+#include <gnome-pilot-structures.h>
+#include <gnome-pilot-conduit.h>
+#include <gnome-pilot-conduit-management.h>
+#include <gnome-pilot-conduit-config.h>
 
 typedef struct _ConduitState ConduitState;
 struct _ConduitState {
@@ -49,7 +49,7 @@ struct _ConduitState {
 	GnomePilotConduitConfig     *config;
 	GnomePilotConduit           *conduit;
 	GtkWidget                   *settings_widget;
-	GtkObject                   *settings_widget2;
+	GObject                     *settings_widget2;
 	GList                       *valid_synctypes;
 };
 
