@@ -21,10 +21,15 @@
  *          Robert Mibus
  */
 
-/* $Id: backup_conduit.c,v 1.47 2003/03/18 15:12:25 jpr Exp $ */
+/* $Id$ */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <stdio.h>
 #include <glib.h>
-#include <gnome.h>
+#include <glib/gi18n.h>
 
 #include <pi-source.h>
 #include <pi-socket.h>
@@ -36,6 +41,7 @@
 
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <ctype.h>
 #include <utime.h>
 #include <unistd.h>
 #include <pwd.h>
@@ -43,7 +49,8 @@
 #include <errno.h>
 #include <dirent.h>
 
-#include <gpilotd/gnome-pilot-conduit-backup.h>
+#include <gnome-pilot-conduit-backup.h>
+#include <gnome-pilot-config.h>
 #include "backup_conduit.h"
 
 #define DEBUG 1
@@ -58,6 +65,7 @@ typedef struct file_db {
 	gint entries;
 } file_db;
 
+#ifdef DEBUG_REC_IO
 static char*
 pi_unmktag (long type) {
 	static char tag[5];
@@ -68,9 +76,13 @@ pi_unmktag (long type) {
 	tag[4] = 0;
 	return tag;
 }
+#endif
 
 GnomePilotConduit *conduit_get_gpilot_conduit (guint32 pilotId);
 void conduit_destroy_gpilot_conduit (GnomePilotConduit *conduit);
+void error_dialog (GtkWindow *parent, gchar *mesg, ...);
+gboolean check_base_directory (const gchar *dir_name);
+GnomePilotConduit *conduit_load_gpilot_conduit (GPilotPilot *pilot);
 
 static void 
 load_configuration(GnomePilotConduit *conduit, 
@@ -78,28 +90,43 @@ load_configuration(GnomePilotConduit *conduit,
 		   GPilotPilot *pilot)
 {
 	gchar *temp_name;
-	gchar *prefix;
+	gchar *iPilot;
 	gchar **exclude_files;
-	gint num_of_exclude_files = 0;
+	gsize num_of_exclude_files = 0;
 	guint i;
 	DIR *dir;
 	struct dirent *entry;
+  	GKeyFile *kfile;
+	GError   *error = NULL;
 
 	(*c) = g_new0(ConduitCfg,1);
 	(*c)->child = -1;
 	
-	prefix = g_strdup_printf(CONFIG_PREFIX, pilot->pilot_id);
-	
-	gnome_config_push_prefix(prefix);
-	(*c)->backup_dir = gnome_config_get_string("backup_dir");
-	(*c)->updated_only = gnome_config_get_bool("updated_only=TRUE");
-	(*c)->remove_deleted = gnome_config_get_bool("remove_deleted=FALSE");	
-	(*c)->no_of_backups = gnome_config_get_int("no_of_backups=0");
+ 	kfile = get_backup_kfile ();
+	iPilot = g_strdup_printf ("Pilot_%u", pilot->pilot_id);
+	(*c)->backup_dir = g_key_file_get_string (kfile, iPilot, "backup_dir", NULL);
+
+	(*c)->updated_only = g_key_file_get_boolean (kfile, iPilot, "updated_only", &error);
+	if (error) {
+		g_warning (_("Unable load key backup-conduit/%s/updated_only: %s"), iPilot, error->message);
+		g_error_free (error);
+		error = NULL;
+		(*c)->updated_only = TRUE;
+	}
+
+	(*c)->remove_deleted = g_key_file_get_boolean (kfile, iPilot, "remove_deleted", NULL);	
+	if (error) {
+		g_warning (_("Unable load key backup-conduit/%s/remove_deleted: %s"), iPilot, error->message);
+		g_error_free (error);
+		error = NULL;
+		(*c)->remove_deleted = FALSE;
+	}
+
+	(*c)->no_of_backups = g_key_file_get_integer (kfile, iPilot, "no_of_backups", NULL);
 
 	(*c)->exclude_files = NULL;
-	gnome_config_get_vector("exclude_files",
-				&num_of_exclude_files,
-				&exclude_files);
+	exclude_files = g_key_file_get_string_list (kfile, iPilot, "exclude_files",
+						    &num_of_exclude_files, NULL);
 	if(num_of_exclude_files) {
 		for( i = 0; i < num_of_exclude_files ; i++ ) {
 			(*c)->exclude_files = g_list_append( (*c)->exclude_files , 
@@ -108,7 +135,6 @@ load_configuration(GnomePilotConduit *conduit,
 		}
 		g_free(exclude_files);
 	}
-	gnome_config_pop_prefix();
 	
 	if ((*c)->backup_dir == NULL) {
 		if (conduit != NULL && GNOME_IS_PILOT_CONDUIT (conduit)) {
@@ -157,21 +183,23 @@ load_configuration(GnomePilotConduit *conduit,
 
 	(*c)->pilotId = pilot->pilot_id;
 
-	g_free(prefix);
+	g_free (iPilot);
+	g_key_file_free (kfile);
 }
 
 static void 
 save_configuration(ConduitCfg *c) 
 {
-	gchar *prefix;
-	gchar **exclude;
-	gchar *exc=NULL;
+	gchar *iPilot;
+	const gchar **exclude;
 	GList *iterator;
-	guint i;
+	guint i = 0;
+	GKeyFile *kfile;
 
 	g_return_if_fail(c!=NULL);
        	
-	prefix= g_strdup_printf("/gnome-pilot.d/backup-conduit/Pilot_%u/",c->pilotId);
+	kfile = get_backup_kfile ();
+	iPilot = g_strdup_printf("Pilot_%u",c->pilotId);
 	
 	if( c->exclude_files != NULL ) {
 	  iterator = c->exclude_files;
@@ -180,20 +208,22 @@ save_configuration(ConduitCfg *c)
 	    exclude[i] = iterator->data;
 	  }
 	  exclude[i] = NULL;
-	  exc = g_strjoinv( ",", exclude);
-	  g_free(exclude);
 	}
-	gnome_config_push_prefix(prefix);
-	gnome_config_set_string("backup_dir",c->backup_dir);
-	gnome_config_set_bool("updated_only",c->updated_only);
-	gnome_config_set_bool("remove_deleted",c->remove_deleted);
-	gnome_config_set_int("no_of_backups",c->no_of_backups);
-	gnome_config_set_string("exclude_files",exc);
-	gnome_config_pop_prefix();
-	gnome_config_sync();
-	gnome_config_drop_all();
-	g_free(prefix);
-	g_free(exc);
+	g_key_file_set_string (kfile, iPilot, "backup_dir",c->backup_dir);
+	g_key_file_set_boolean (kfile, iPilot, "updated_only",c->updated_only);
+	g_key_file_set_boolean (kfile, iPilot, "remove_deleted",c->remove_deleted);
+	g_key_file_set_integer (kfile, iPilot, "no_of_backups",c->no_of_backups);
+	if (i != 0) {
+		g_key_file_set_string_list (kfile, iPilot, "exclude_files", exclude, i);
+		g_free(exclude);
+	} else {
+		g_key_file_set_string (kfile, iPilot, "exclude_files", "");
+	}
+
+	g_free(iPilot);
+
+	save_backup_kfile (kfile);
+	g_key_file_free (kfile);
 }
 
 static void 
@@ -237,13 +267,15 @@ destroy_configuration(ConduitCfg **c)
 		for (iterator = (*c)->files_in_backup; iterator; iterator = g_list_next (iterator)) {
 			gchar *filename = (char*)iterator->data;
 			gchar *backup_dirname;
+			gchar *basename;
 			gchar *backup_backup;
 			
 			/* I18N note: this is printed when renaming a file %s */
 			g_message (_("Renaming %s"), filename);
 
-			backup_dirname = g_dirname (filename);
-			backup_backup = g_strdup_printf ("%s/del/%s", backup_dirname, g_basename (filename));
+			backup_dirname = g_path_get_dirname (filename);
+			basename = g_path_get_basename (filename);
+			backup_backup = g_strdup_printf ("%s/del/%s", backup_dirname, basename);
 
 			/* I18N note: this message follow a "Renaming %s". The
 			   %s is a file name */
@@ -253,6 +285,7 @@ destroy_configuration(ConduitCfg **c)
 			}
 
 			g_free (backup_backup);
+			g_free (basename);
 			g_free (backup_dirname);
 			g_free (filename);
 		}
@@ -263,6 +296,7 @@ destroy_configuration(ConduitCfg **c)
 	g_list_foreach ((*c)->exclude_files,(GFunc)g_free,NULL);
 	g_list_free ((*c)->exclude_files);
 	g_free ((*c)->backup_dir);
+	g_free ((*c)->old_backup_dir);
 	g_free (*c);
 	*c = NULL;
 }
@@ -298,7 +332,7 @@ gnome_pilot_conduit_backup_remove_deleted (GnomePilotConduitBackup *conduit,
 		GList *iterator;
 		for (iterator = cfg->files_in_backup; iterator; iterator = g_list_next (iterator)) {
 			char *filename = (char*)iterator->data;
- 			if (filename && strcasecmp(filename, name) == 0) {
+			if (filename && g_ascii_strcasecmp (filename, name) == 0) {
 				cfg->files_in_backup = g_list_remove_link (cfg->files_in_backup, 
 									   iterator);
 				g_free (filename);
@@ -377,7 +411,7 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 					gpointer _cfg)
 {
 	char *name;
-	struct pi_file *f;
+	struct pi_file *f = NULL;
 	struct stat statb;
 	struct utimbuf times;
 	GList *iterator;
@@ -387,9 +421,10 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 	int keep_reading;
 	int entries;
 	int err;
-	unsigned char *buffer;
 	PilotRecord remote;
 	int wrote;
+	pi_buffer_t *piBuf = NULL;
+	int len;
 
 	g_return_val_if_fail (conduit != NULL, -1);
 	g_return_val_if_fail (dbinfo != NULL, -1);
@@ -397,7 +432,7 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 	
 	/* let's see first if we should actually touch this db */
 	for (iterator = cfg->exclude_files; iterator; iterator = g_list_next (iterator)) {
-		if( ! g_strcasecmp(iterator->data, PI_DBINFO (dbinfo)->name) ) {
+		if(!g_ascii_strcasecmp (iterator->data, PI_DBINFO (dbinfo)->name)) {
 			g_message("excluded %s",PI_DBINFO (dbinfo)->name);
 			result = 2;
 			goto exit;
@@ -443,7 +478,7 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 
 	/* backup new db */
 	f = pi_file_create (name, PI_DBINFO (dbinfo));
-	if(f==0) {
+	if(f == NULL) {
 		g_warning(_("Could not create backup file %s"),name);
 		gnome_pilot_conduit_send_error (GNOME_PILOT_CONDUIT(conduit),
 						_("Could not create backup file %s"),name);
@@ -460,7 +495,10 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 	if (err < 0) {
 		g_warning ("error (%s) in opening '%s'", dlp_strerror (err), PI_DBINFO (dbinfo)->name);
 		result = -1;
-		goto db_close;
+		/* DB not successfully opened, so don't try and close (caused
+		 * error on Tungsten E2) probably cause of Ubuntu bug #81396
+		 */
+		goto exit_and_free; 
 	}
 
 	err = dlp_ReadOpenDBInfo (dbinfo->pilot_socket, 
@@ -471,23 +509,18 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 		g_warning ("error (%s) in reading '%s'", dlp_strerror (err), PI_DBINFO (dbinfo)->name);
 		result = -1;
 		goto db_close;
-	}		
-
-	buffer = g_new0 (unsigned char, 0xffff);
-	remote.buffer = buffer;
-
-	{
-		int l;
-		l = dlp_ReadAppBlock (dbinfo->pilot_socket,
-				      dbinfo->db_handle,
-				      0,
-				      buffer,
-				      0xffff);
-		if (l>0) {
-			pi_file_set_app_info (f, buffer, l);
-		}
 	}
-	
+
+
+       piBuf = pi_buffer_new (0xffff);
+       len = dlp_ReadAppBlock (dbinfo->pilot_socket,
+                               dbinfo->db_handle,
+                               0,
+                               -1,
+                               piBuf);
+
+       if (len > 0)
+               pi_file_set_app_info (f, piBuf->data, len);
 
 	index = 0;
 	keep_reading = 1;
@@ -500,19 +533,19 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 			keep_reading = (dlp_ReadResourceByIndex (dbinfo->pilot_socket, 
 								 dbinfo->db_handle,
 								 index, 
-								 remote.buffer,
+								 piBuf,
 								 &type,
-								 &id,
-								 &remote.len) >= 0);
+								 &id) >= 0);
+
 #ifdef DEBUG_REC_IO
 			g_message ("read resource %d, type = %s, size %d, , index %d/%d",
-				   id, pi_unmktag (type), remote.len,
+				   id, pi_unmktag (type), piBuf->used,
 				   index, entries);
 #endif
-			if (keep_reading) {				
-				err = pi_file_append_resource (f, 
-							       remote.buffer, 
-							       remote.len, 
+			if (keep_reading > 0) {
+				err = pi_file_append_resource (f,
+							       piBuf->data,
+							       piBuf->used,
 							       type,
 							       id);
 				if (err < 0) {
@@ -521,7 +554,7 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 					wrote++;
 #ifdef DEBUG_REC_IO
 					g_message ("write resource %d, type = %s, size %d, index %d/%d",
-						   id, pi_unmktag (type), remote.len,
+						   id, pi_unmktag (type), piBuf->used,
 						   index, entries);
 #endif
 				}
@@ -530,22 +563,21 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 			keep_reading = (dlp_ReadRecordByIndex (dbinfo->pilot_socket, 
 							       dbinfo->db_handle,
 							       index, 
-							       remote.buffer,
+							       piBuf,
 							       &remote.recID,
-							       &remote.len,
 							       &remote.flags,
 							       &remote.catID) >= 0);
 
 #ifdef DEBUG_REC_IO
 			g_message ("read record %d, size %d, index %d/%d",
-				   remote.recID, remote.len,
+				   remote.recID, piBuf->used,
 				   index, entries);
 #endif
 
-			if (keep_reading) {
+			if (keep_reading > 0) {
 				err = pi_file_append_record (f, 
-							     remote.buffer, 
-							     remote.len, 
+							     piBuf->data, 
+							     piBuf->used, 
 							     remote.flags,
 							     remote.catID,
 							     remote.recID);
@@ -555,7 +587,7 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 					wrote++;
 #ifdef DEBUG_REC_IO
 					g_message ("write record %d, size %d, index %d/%d",
-						   id, remote.len,
+						   id, piBuf->used,
 						   index, entries);
 #endif
 				}
@@ -567,17 +599,17 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 		gnome_pilot_conduit_send_progress (GNOME_PILOT_CONDUIT (conduit),
 						   entries,
 						   index);
-	} while (keep_reading && index < entries);
+	} while ((keep_reading > 0) && (index < entries));
 
-	g_free (buffer);
-		
 	if (pi_file_close (f) < 0) {
 		g_warning("backup conduit can't write file");
 		gnome_pilot_conduit_send_error(GNOME_PILOT_CONDUIT(conduit),
 					       "Unable to write file to backup directory specified");
 		result = -1;
+		f = NULL;
 		goto db_close;
 	}
+	f = NULL;
 	
 	g_message ("Wrote %d of %d %s, which is %s",
 		   wrote, entries, PI_DBINFO (dbinfo)->flags & dlpDBFlagResource ? "resources" : "records",
@@ -589,7 +621,12 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
  db_close:
 	dlp_CloseDB (dbinfo->pilot_socket, dbinfo->db_handle);
  exit_and_free:
+	if(f != NULL)
+		pi_file_close(f);
 	g_free (name);
+	if (piBuf) {
+		pi_buffer_free (piBuf);
+	}
  exit:	
 	return result;	
 }
@@ -620,15 +657,16 @@ gnome_real_pilot_conduit_backup_restore (GnomePilotConduitBackup *conduit,
 					 gpointer _cfg)
 {
 	DIR *dir;
+	struct stat buf;
 	struct dirent *dirent;
-	GnomePilotDBInfo info;
 	file_db **db;
 	int dbcount = 0;
-	int i, j, max, size;
+	int i, j;
+	size_t size;
 	struct pi_file *file;
 	char *source;
 	int result = 0;
-	int reset = 0;
+
 	ConduitCfg *cfg = (ConduitCfg*)_cfg;
 
 	if (src_dir == NULL) {
@@ -652,8 +690,11 @@ gnome_real_pilot_conduit_backup_restore (GnomePilotConduitBackup *conduit,
 		printf ("checking %s/%s\n", source, dirent->d_name);
 #endif
 
-		/* skip dotfiles. */
-		if (dirent->d_name[0] == '.')
+		gchar *entryname = g_strdup_printf("%s/%s", source, dirent->d_name);
+		/* skip dotfiles and directories. */
+		lstat (entryname, &buf);
+		g_free(entryname);
+		if (dirent->d_name[0] == '.' || S_ISDIR (buf.st_mode))
 			continue;
 
 		db[dbcount] = g_new0 (file_db, 1);
@@ -679,7 +720,7 @@ gnome_real_pilot_conduit_backup_restore (GnomePilotConduitBackup *conduit,
 
 		/* Find biggest record */
 		for (i = 0; i < db[dbcount]->entries; i++) {
-			if (PI_DBINFO (&info)->flags & dlpDBFlagResource)
+			if (db[dbcount]->info.flags & dlpDBFlagResource)
 				pi_file_read_resource(file, i, 0, &size, 0, 0);
 			else
 				pi_file_read_record(file, i, 0, &size, 0, 0,0 );
@@ -711,7 +752,6 @@ gnome_real_pilot_conduit_backup_restore (GnomePilotConduitBackup *conduit,
 		}
         }
 
- exit_and_free:
 #ifdef DEBUG
 	fprintf (stderr,"backup_conduit: Restore done\n");
 #endif
@@ -721,7 +761,7 @@ gnome_real_pilot_conduit_backup_restore (GnomePilotConduitBackup *conduit,
 	}
 
 	g_free(db);
- exit:
+
 	return result;
 }
 
@@ -738,13 +778,13 @@ insert_dir_callback (GtkEditable *editable, const gchar *text,
 	curname = gtk_entry_get_text(GTK_ENTRY(editable));
 	if (*curname == '\0' && len > 0) {
 		if (isspace(text[0])) {
-			gtk_signal_emit_stop_by_name(GTK_OBJECT(editable), "insert_text");
+			g_signal_stop_emission_by_name (G_OBJECT(editable), "insert_text");
 			return;
 		}
 	} else {
 		for (i=0; i<len; i++) {
 			if (isspace(text[i])) {
-				gtk_signal_emit_stop_by_name(GTK_OBJECT(editable), 
+				g_signal_stop_emission_by_name (G_OBJECT(editable), 
 							     "insert_text");
 				return;
 			}
@@ -758,67 +798,153 @@ static GtkWidget
 	GtkWidget *vbox, *table;
 	GtkWidget *entry, *label;
 	GtkWidget *button, *spin;
-	GtkObject *adjustment;
+	GObject *adjustment;
+	AtkObject *atk_widget;
 
-	vbox = gtk_vbox_new(FALSE, GNOME_PAD);
+	vbox = gtk_vbox_new (FALSE, 8);
 
 	table = gtk_table_new(2, 4, FALSE);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 10);
-	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, GNOME_PAD);
+	gtk_box_pack_start (GTK_BOX(vbox), table, FALSE, FALSE, 8);
 
-	label = gtk_label_new(_("Backup directory"));
+	label = gtk_label_new_with_mnemonic(_("_Backup directory:"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
     
-	entry = gtk_entry_new_with_max_length(128);
-	gtk_object_set_data(GTK_OBJECT(vbox), "dir", entry);
+	entry = gtk_entry_new ();
+	gtk_entry_set_max_length(GTK_ENTRY(entry), 128);
+	g_object_set_data (G_OBJECT(vbox), "dir", entry);
 	gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 0, 1);
-	gtk_signal_connect(GTK_OBJECT(entry), "insert_text",
-			   GTK_SIGNAL_FUNC(insert_dir_callback),
+	g_signal_connect(G_OBJECT(entry), "insert_text",
+			   G_CALLBACK(insert_dir_callback),
 			   NULL);
 
-	label = gtk_label_new(_("Only backup changed bases"));
+	gtk_label_set_mnemonic_widget (GTK_LABEL(label), entry);
+	atk_widget = gtk_widget_get_accessible (entry);
+	atk_object_set_name (atk_widget, _("Backup directory"));
+
+	label = gtk_label_new_with_mnemonic (_("O_nly backup changed bases"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1,2);
 
 	button = gtk_check_button_new();
-	gtk_object_set_data(GTK_OBJECT(vbox), "only_changed", button);
+	g_object_set_data (G_OBJECT(vbox), "only_changed", button);
 	gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 1,2);
 
-	label = gtk_label_new(_("Remove local base if deleted on pilot"));
+	gtk_label_set_mnemonic_widget (GTK_LABEL(label), button);
+	atk_widget = gtk_widget_get_accessible (button);
+	atk_object_set_name (atk_widget, _("Only backup changed bases"));
+
+	label = gtk_label_new_with_mnemonic(_("_Remove local base if deleted on pilot"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2,3);
 
 	button = gtk_check_button_new();
-	gtk_object_set_data(GTK_OBJECT(vbox), "remove_local", button);
+	g_object_set_data (G_OBJECT(vbox), "remove_local", button);
 	gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 2,3);
 
-	label = gtk_label_new(_("# of old backups to keep"));
+	gtk_label_set_mnemonic_widget (GTK_LABEL(label), button);
+	atk_widget = gtk_widget_get_accessible (button);
+	atk_object_set_name (atk_widget, _("Remove local base if deleted on pilot"));
+
+	label = gtk_label_new_with_mnemonic (_("# of old backups to _keep"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 3,4);
 
-	adjustment = gtk_adjustment_new (0, 0, 100, 1, 1, 1);
+	adjustment = G_OBJECT(gtk_adjustment_new (0, 0, 100, 1, 1, 1));
 	spin = gtk_spin_button_new (GTK_ADJUSTMENT(adjustment), 1, 0);
-	gtk_object_set_data(GTK_OBJECT(vbox), "no_of_backups", adjustment);
+	g_object_set_data (G_OBJECT(vbox), "no_of_backups", adjustment);
 	gtk_table_attach_defaults(GTK_TABLE(table), spin, 1, 2, 3, 4);
 
+	gtk_label_set_mnemonic_widget (GTK_LABEL(label), spin);
+	atk_widget = gtk_widget_get_accessible (spin);
+	atk_object_set_name (atk_widget, _("# of old backups to keep"));
+
 	return vbox;
 }
 
+void
+error_dialog (GtkWindow *parent, gchar *mesg, ...)
+{
+        GtkWidget *dlg;
+        char *tmp;
+        va_list ap;
+
+        va_start (ap,mesg);
+        tmp = g_strdup_vprintf (mesg,ap);
+
+        dlg = gtk_message_dialog_new (parent, GTK_DIALOG_DESTROY_WITH_PARENT,
+	    GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "%s", tmp);
+        gtk_dialog_run (GTK_DIALOG (dlg));
+        gtk_widget_destroy (dlg);
+
+        va_end (ap);
+        g_free (tmp);
+}
+
+gboolean
+check_base_directory (const gchar *dir_name)
+{
+        gboolean ret = TRUE;
+        /* check basedir validity */
+
+        if (mkdir (dir_name, 0700) < 0 ) {
+                struct stat buf;
+                gchar *errstr;
+                switch (errno) {
+                case EEXIST:
+                        stat (dir_name, &buf);
+                        if (S_ISDIR (buf.st_mode)) {
+                                if (!(buf.st_mode & (S_IRUSR | S_IWUSR |S_IXUSR))) {
+                                        error_dialog (NULL, _("The specified backup directory exists but has the wrong permissions.\n"
+                                                        "Please fix or choose another directory"));
+                                        ret = FALSE;
+                                }
+                        } else {
+                                error_dialog (NULL, _("The specified backup directory exists but is not a directory.\n"
+                                                "Please make it a directory or choose another directory"));
+                                ret = FALSE;
+                        }
+                        break;
+
+                case EACCES:
+                        error_dialog (NULL, _("It wasn't possible to create the specified backup directory.\n"
+                                        "Please verify the permissions on the specified path or choose another directory"));
+                        ret = FALSE;
+			break;
+                case ENOENT:
+                        error_dialog (NULL, _("The path specified for the backup directory is invalid.\n"
+                                        "Please choose another directory"));
+                        ret = FALSE;
+                        break;
+                default:
+                        errstr = strerror (errno);
+                        error_dialog (NULL, errstr);
+                        ret = FALSE;
+                }
+        }
+        return ret;
+}
+
 static void
 setOptionsCfg(GtkWidget *pilotcfg, ConduitCfg *state)
 {
 	GtkWidget *dir,*updated_only,*remove_deleted;
-	GtkObject *adj;
+	GObject *adj;
 
-	dir  = gtk_object_get_data(GTK_OBJECT(pilotcfg), "dir");
-	updated_only = gtk_object_get_data(GTK_OBJECT(pilotcfg), "only_changed");
-	remove_deleted = gtk_object_get_data(GTK_OBJECT(pilotcfg), "remove_local");
-	adj = gtk_object_get_data(GTK_OBJECT(pilotcfg), "no_of_backups");
+	dir  = g_object_get_data (G_OBJECT(pilotcfg), "dir");
+	updated_only = g_object_get_data (G_OBJECT(pilotcfg), "only_changed");
+	remove_deleted = g_object_get_data (G_OBJECT(pilotcfg), "remove_local");
+	adj = g_object_get_data (G_OBJECT(pilotcfg), "no_of_backups");
 
 	g_assert(dir!=NULL);
 	g_assert(updated_only!=NULL);
 	g_assert(remove_deleted!=NULL);
 	g_assert(adj!=NULL);
 
-	gtk_entry_set_text(GTK_ENTRY(dir), state->backup_dir);
+	state->old_backup_dir = g_strdup(state->backup_dir);
+	gtk_entry_set_text(GTK_ENTRY(dir), state->old_backup_dir);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(updated_only), state->updated_only);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(remove_deleted), state->remove_deleted);
 	gtk_adjustment_set_value (GTK_ADJUSTMENT(adj), state->no_of_backups);
@@ -829,17 +955,24 @@ static void
 readOptionsCfg(GtkWidget *pilotcfg, ConduitCfg *state)
 {
 	GtkWidget *dir,*updated_only,*remove_deleted;
-	GtkObject *adj;
+	GObject *adj;
 
-	dir  = gtk_object_get_data(GTK_OBJECT(pilotcfg), "dir");
-	updated_only = gtk_object_get_data(GTK_OBJECT(pilotcfg), "only_changed");
-	remove_deleted = gtk_object_get_data(GTK_OBJECT(pilotcfg), "remove_local");
-	adj = gtk_object_get_data(GTK_OBJECT(pilotcfg), "no_of_backups");
+	dir  = g_object_get_data (G_OBJECT(pilotcfg), "dir");
+	updated_only = g_object_get_data (G_OBJECT(pilotcfg), "only_changed");
+	remove_deleted = g_object_get_data (G_OBJECT(pilotcfg), "remove_local");
+	adj = g_object_get_data (G_OBJECT(pilotcfg), "no_of_backups");
 
-	state->backup_dir = g_strdup(gtk_entry_get_text(GTK_ENTRY(dir)));
+	if(state->backup_dir)
+		g_free(state->backup_dir);	
+	if(check_base_directory ( gtk_entry_get_text (GTK_ENTRY (dir)))){
+		state->backup_dir = g_strdup(gtk_entry_get_text(GTK_ENTRY(dir)));
+	} else {
+		state->backup_dir = g_strdup(state->old_backup_dir);
+		gtk_entry_set_text(GTK_ENTRY(dir), state->old_backup_dir);
+	}
 	state->updated_only = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(updated_only));
 	state->remove_deleted = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(remove_deleted));
-	state->no_of_backups = GTK_ADJUSTMENT(adj)->value;
+	state->no_of_backups = gtk_adjustment_get_value(GTK_ADJUSTMENT(adj));
 }
 
 static gint
@@ -851,7 +984,7 @@ create_settings_window (GnomePilotConduit *conduit, GtkWidget *parent, gpointer
 	gtk_container_add(GTK_CONTAINER(parent),cfgWindow);
 	gtk_widget_show_all(cfgWindow);
 
-	gtk_object_set_data(GTK_OBJECT(conduit),OBJ_DATA_CONFIG_WINDOW,cfgWindow);
+	g_object_set_data (G_OBJECT(conduit),OBJ_DATA_CONFIG_WINDOW,cfgWindow);
 	setOptionsCfg(GET_CONDUIT_WINDOW(conduit),GET_CONDUIT_CFG(conduit));
 
 	return 0;
@@ -886,19 +1019,19 @@ GnomePilotConduit *
 conduit_load_gpilot_conduit (GPilotPilot *pilot)
 {
 	ConduitCfg *cfg, *cfg2;
-	GtkObject *retval;
+	GnomePilotConduitBackup *retval;
 
-	retval = gnome_pilot_conduit_backup_new (pilot);
+	retval = GNOME_PILOT_CONDUIT_BACKUP(gnome_pilot_conduit_backup_new (pilot));
 	g_assert (retval != NULL);
 
 	load_configuration(GNOME_PILOT_CONDUIT (retval), &cfg, pilot);
 
 	cfg2 = dupe_configuration(cfg);
 
-	gtk_object_set_data(GTK_OBJECT(retval),OBJ_DATA_CONFIG,cfg);
-	gtk_object_set_data(GTK_OBJECT(retval),OBJ_DATA_OLDCONFIG,cfg2);
+	g_object_set_data (G_OBJECT(retval),OBJ_DATA_CONFIG,cfg);
+	g_object_set_data (G_OBJECT(retval),OBJ_DATA_OLDCONFIG,cfg2);
 
-	gtk_object_set_data(retval,
+	g_object_set_data (G_OBJECT(retval),
 			    "configuration",
 			    cfg);
 
@@ -906,14 +1039,14 @@ conduit_load_gpilot_conduit (GPilotPilot *pilot)
 					    cfg);
 	gnome_pilot_conduit_backup_connect__restore (retval, gnome_real_pilot_conduit_backup_restore, 
 					     cfg);
-	gnome_pilot_conduit_connect__create_settings_window (retval, create_settings_window, 
-							    NULL);
-	gnome_pilot_conduit_connect__display_settings (retval, display_settings, 
-						      NULL);
-	gnome_pilot_conduit_connect__save_settings (retval, save_settings, 
-						   NULL);
-	gnome_pilot_conduit_connect__revert_settings (retval, revert_settings, 
-						     NULL);	
+	gnome_pilot_conduit_connect__create_settings_window (GNOME_PILOT_CONDUIT(retval),
+	    create_settings_window, NULL);
+	gnome_pilot_conduit_connect__display_settings (GNOME_PILOT_CONDUIT(retval),
+	    display_settings, NULL);
+	gnome_pilot_conduit_connect__save_settings (GNOME_PILOT_CONDUIT(retval),
+	    save_settings, NULL);
+	gnome_pilot_conduit_connect__revert_settings (GNOME_PILOT_CONDUIT(retval),
+	    revert_settings, NULL);	
 
 	return GNOME_PILOT_CONDUIT (retval);
 }
@@ -924,6 +1057,6 @@ conduit_destroy_gpilot_conduit (GnomePilotConduit *conduit)
 	ConduitCfg *cfg;
 	cfg = GET_CONDUIT_CFG(conduit);
 	destroy_configuration(&cfg);
-	gtk_object_destroy (GTK_OBJECT (conduit));
+	g_object_unref (G_OBJECT (conduit));
 }
 
