@@ -21,30 +21,23 @@
  */
 
 #include "gpilot-gui.h"
-#include <gnome.h>
+#include <string.h>
+#include <stdlib.h>
+#include <gtk/gtk.h>
 
-void 
-gpilot_gui_run_dialog(gchar *type,gchar *mesg,...)
+static void gpilot_gui_run_dialog(GtkMessageType type, gchar *mesg, va_list ap);
+
+static void
+gpilot_gui_run_dialog(GtkMessageType type, gchar *mesg, va_list ap)
 {
 	char *tmp;
-	GtkWidget *dialogWindow;
-	va_list ap;
-	va_start(ap,mesg);
+	GtkWidget *dialog;
 
 	tmp = g_strdup_vprintf(mesg,ap);
-#if 0
-	dialogWindow = gnome_message_box_new(tmp,type,GNOME_STOCK_BUTTON_OK,NULL);
-	gnome_dialog_run_and_close(GNOME_DIALOG(dialogWindow));
-#endif
-
-	if (strcmp(type,GNOME_MESSAGE_BOX_WARNING)==0) {
-		gnome_warning_dialog(tmp);
-	} else if (strcmp(type,GNOME_MESSAGE_BOX_ERROR)==0) {
-		dialogWindow = gnome_message_box_new(tmp,type,GNOME_STOCK_BUTTON_OK,NULL);
-		gnome_dialog_run(GNOME_DIALOG(dialogWindow));
-		exit(-1);
-	}
 
+	dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, type, GTK_BUTTONS_OK, "%s", tmp);
+	gtk_dialog_run(GTK_DIALOG(dialog));
+	gtk_widget_destroy(dialog);
 	g_free(tmp);
 	va_end(ap);
 }
@@ -55,7 +48,7 @@ gpilot_gui_warning_dialog (gchar *mesg, ...)
 	va_list args;
 
 	va_start (args, mesg);
-	gpilot_gui_run_dialog (GNOME_MESSAGE_BOX_WARNING, mesg, args);
+	gpilot_gui_run_dialog (GTK_MESSAGE_WARNING, mesg, args);
 	va_end (args);
 }
 
@@ -65,24 +58,10 @@ gpilot_gui_error_dialog (gchar *mesg, ...)
 	va_list args;
 
 	va_start (args, mesg);
-	gpilot_gui_run_dialog (GNOME_MESSAGE_BOX_ERROR, mesg, args);
+	gpilot_gui_run_dialog (GTK_MESSAGE_ERROR, mesg, args);
 	va_end (args);
 }
 
-static void
-gpilot_gui_restore_callback (gint reply,
-			     gpointer data)
-{
-	gboolean *result = (gboolean*)data;
-	if (reply == 0 /* YES */) {
-		(*result) = TRUE;
-	} else if (reply == 1 /* NO */) {
-		(*result) = FALSE;
-	} else {
-		g_assert_not_reached ();
-	}
-}
-
 GPilotPilot* 
 gpilot_gui_restore (GPilotContext *context, 
 		    GPilotPilot *pilot)
@@ -91,20 +70,19 @@ gpilot_gui_restore (GPilotContext *context,
 	GtkWidget *d;
 
 	if (pilot) {
-		gboolean q_result = FALSE;
+		gint response;
 		char *tmp;
 		tmp = g_strdup_printf ("Restore %s' pilot with id %d\n"
 				       "and name `%s'",
 				       pilot->pilot_username,
 				       pilot->pilot_id, 
 				       pilot->name);
-		d = gnome_question_dialog_modal (tmp,
-						 gpilot_gui_restore_callback,
-						 GINT_TO_POINTER (&q_result));
-		gnome_dialog_run (GNOME_DIALOG (d));
+		d = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION,
+		    GTK_BUTTONS_YES_NO, "%s", tmp);
+		response = gtk_dialog_run (GTK_DIALOG (d));
 		g_free (tmp);
-
-		if (q_result == TRUE) {
+		gtk_widget_destroy(d);
+		if (response == GTK_RESPONSE_YES) {
 			result = pilot;
 		} else {
 			
