@@ -21,6 +21,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <glib.h>
 #include <gmodule.h>
 #include <pi-socket.h>
@@ -29,9 +33,9 @@
 #include <pi-sync.h>
 #include <pi-dlp.h>
 
-#include <libgnome/libgnome.h>
+#include <glib/gi18n.h>
 
-#include "orbit_daemon_glue.h"
+#include "gpilot-daemon.h"
 #include "manager.h"
 #include "queue_io.h"
 #include "gnome-pilot-conduit-backup.h"
@@ -40,9 +44,10 @@
 #include "gnome-pilot-structures.h"
 #include "gnome-pilot-dbinfo.h"
 #include "gpilot-gui.h"
+#include "gnome-pilot-config.h"
 
-#include <libgpilotdCM/gnome-pilot-conduit-management.h>
-#include <libgpilotdCM/gnome-pilot-conduit-config.h>
+#include "gnome-pilot-conduit-management.h"
+#include "gnome-pilot-conduit-config.h"
 
 #define GPCM_LABEL "conduit_mgmt"
 #define GPCC_LABEL "conduit_cfg"
@@ -158,7 +163,7 @@ find_matching_conduit_name (GList *conlist, gchar *name)
 
 /**************************************************************************/
 /* Here comes conduit signal handler, used to call methods from 
-   orbit_daemon_glue                                                      */
+   gpilot-daemon                                                      */
 
 typedef struct {
 	GnomePilotInstallCarrier *carrier;
@@ -171,7 +176,7 @@ gnome_pilot_conduit_backup_func (gchar *filename,
 				 gint db,
 				 gint total_dbs,
 				 GnomePilotBackupFuncCarrier *carrier) {
-	orbed_notify_overall_progress (carrier->pilot_name, db, total_dbs);
+	dbus_notify_overall_progress (carrier->pilot_name, db, total_dbs);
 	
 	g_message ("gnome_pilot_conduit_backup_func (%s)", filename);
 
@@ -186,7 +191,7 @@ conduit_progress_callback (GnomePilotConduit *conduit,
 			   gint total,
 			   gint current,
 			   gpointer pilot_id) {
-	orbed_notify_conduit_progress ((gchar*)pilot_id,
+	dbus_notify_conduit_progress ((gchar*)pilot_id,
 				       conduit,
 				       current,
 				       total);
@@ -196,7 +201,7 @@ static void
 conduit_message_callback (GnomePilotConduit *conduit, 
 			  gchar *message,
 			  gpointer pilot_id) {
-	orbed_notify_conduit_message ((gchar*)pilot_id,
+	dbus_notify_conduit_message ((gchar*)pilot_id,
 				      conduit,
 				      message);
 }
@@ -205,7 +210,7 @@ static void
 conduit_error_callback (GnomePilotConduit *conduit, 
 			gchar *message,
 			gpointer pilot_id) {
-	orbed_notify_conduit_error ((gchar*)pilot_id,
+	dbus_notify_conduit_error ((gchar*)pilot_id,
 				    conduit,
 				    message);
 }
@@ -243,9 +248,9 @@ set_callbacks (gboolean set, GnomePilotConduit *conduit, gchar *pilot_name)
 				   __FILE__,__LINE__);
 		}
 		
-		gtk_signal_disconnect (GTK_OBJECT (conduit), progress_h);
-		gtk_signal_disconnect (GTK_OBJECT (conduit), error_h);
-		gtk_signal_disconnect (GTK_OBJECT (conduit), message_h);
+		g_signal_handler_disconnect (G_OBJECT (conduit), progress_h);
+		g_signal_handler_disconnect (G_OBJECT (conduit), error_h);
+		g_signal_handler_disconnect (G_OBJECT (conduit), message_h);
 		progress_h = 0;
 		error_h    = 0;
 		message_h  = 0;
@@ -294,22 +299,29 @@ iterate_dbs (gint pfd,
 	GPilotPilot *pilot_info;
 	int error = 0;
 	int index = 0;
-       
+	int result;
+
 	pilot_info = gpilot_find_pilot_by_id (pu->userID, context->pilots);
 
-	orbed_notify_daemon_message (pilot_info->name, NULL, _("Collecting synchronization info..."));
+	dbus_notify_daemon_message (pilot_info->name, NULL, _("Collecting synchronization info..."));
 
 	while (1) {
 		GnomePilotDBInfo *dbinfo;
-		
+		pi_buffer_t *pi_buf;
 		dbinfo = g_new0 (GnomePilotDBInfo, 1);
+
+		pi_buf = pi_buffer_new (sizeof (struct DBInfo));
+		result = dlp_ReadDBList (pfd, 0, dlpDBListRAM, index, pi_buf);
 		/* load next dbinfo block */
-		if (dlp_ReadDBList (pfd, 0, dlpDBListRAM, index, PI_DBINFO (dbinfo)) < 0) {
+		if (result < 0) {
 			/* is <0, there are no more databases, break
                            out so we can save the list */
+			g_free (dbinfo);
 			break;
 		}
 
+		memcpy (dbinfo, pi_buf->data, sizeof (struct DBInfo));
+		pi_buffer_free (pi_buf);
 		index = PI_DBINFO (dbinfo)->index + 1;
 
 		dbs = g_list_append (dbs, dbinfo);
@@ -360,7 +372,7 @@ iterate_dbs (gint pfd,
 				set_callbacks (FALSE, GNOME_PILOT_CONDUIT (conduit), pilot_info->name);
 			}
 
-			orbed_notify_overall_progress (pilot_info->name, index, g_list_length (dbs)); 
+			dbus_notify_overall_progress (pilot_info->name, index, g_list_length (dbs)); 
 
 			/*
 			  It seems pretty erratic which has this and which hasn't,
@@ -407,11 +419,11 @@ conduit_synchronize (GnomePilotConduitStandard *conduit, GnomePilotDBInfo *dbinf
 
 	write_sync_type_line_to_pilot (dbinfo, GNOME_PILOT_CONDUIT (conduit), _("Synchronizing %s\n"));
 
-	orbed_notify_conduit_start (dbinfo->pilotInfo->name,
+	dbus_notify_conduit_start (dbinfo->pilotInfo->name,
 				   GNOME_PILOT_CONDUIT (conduit),
 				   GnomePilotConduitSyncTypeSynchronize);
 	err = gnome_pilot_conduit_standard_synchronize (conduit, dbinfo);
-	orbed_notify_conduit_end (dbinfo->pilotInfo->name,
+	dbus_notify_conduit_end (dbinfo->pilotInfo->name,
 				   GNOME_PILOT_CONDUIT (conduit));
 	return err;
 }
@@ -424,13 +436,13 @@ conduit_copy_to_pilot (GnomePilotConduitStandard *conduit, GnomePilotDBInfo *dbi
 	g_return_val_if_fail (conduit != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PILOT_CONDUIT_STANDARD (conduit), -1);
 
-	write_sync_type_line_to_pilot (dbinfo, GNOME_PILOT_CONDUIT (conduit), _("Copy to pilot %s\n"));
+	write_sync_type_line_to_pilot (dbinfo, GNOME_PILOT_CONDUIT (conduit), _("Copy to PDA %s\n"));
 
-	orbed_notify_conduit_start (dbinfo->pilotInfo->name,
+	dbus_notify_conduit_start (dbinfo->pilotInfo->name,
 				    GNOME_PILOT_CONDUIT (conduit),
 				    GnomePilotConduitSyncTypeCopyToPilot);
 	err = gnome_pilot_conduit_standard_copy_to_pilot (conduit, dbinfo);
-	orbed_notify_conduit_end (dbinfo->pilotInfo->name,
+	dbus_notify_conduit_end (dbinfo->pilotInfo->name,
 				   GNOME_PILOT_CONDUIT (conduit));
 	return err;
 }
@@ -442,13 +454,13 @@ conduit_copy_from_pilot (GnomePilotConduitStandard *conduit, GnomePilotDBInfo *d
 	g_return_val_if_fail (conduit != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PILOT_CONDUIT_STANDARD (conduit), -1);
 
-	write_sync_type_line_to_pilot (dbinfo, GNOME_PILOT_CONDUIT (conduit), _("Copy from pilot %s\n"));
+	write_sync_type_line_to_pilot (dbinfo, GNOME_PILOT_CONDUIT (conduit), _("Copy from PDA %s\n"));
 
-	orbed_notify_conduit_start (dbinfo->pilotInfo->name,
+	dbus_notify_conduit_start (dbinfo->pilotInfo->name,
 				   GNOME_PILOT_CONDUIT (conduit),
 				   GnomePilotConduitSyncTypeCopyFromPilot);
 	err = gnome_pilot_conduit_standard_copy_from_pilot (conduit, dbinfo);
-	orbed_notify_conduit_end (dbinfo->pilotInfo->name,
+	dbus_notify_conduit_end (dbinfo->pilotInfo->name,
 				   GNOME_PILOT_CONDUIT (conduit));
 	return err;
 }
@@ -460,13 +472,13 @@ conduit_merge_to_pilot (GnomePilotConduitStandard *conduit, GnomePilotDBInfo *db
 	g_return_val_if_fail (conduit != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PILOT_CONDUIT_STANDARD (conduit), -1);
 
-	write_sync_type_line_to_pilot (dbinfo, GNOME_PILOT_CONDUIT (conduit), _("Merge to pilot %s\n"));
+	write_sync_type_line_to_pilot (dbinfo, GNOME_PILOT_CONDUIT (conduit), _("Merge to PDA %s\n"));
 
-	orbed_notify_conduit_start (dbinfo->pilotInfo->name,
+	dbus_notify_conduit_start (dbinfo->pilotInfo->name,
 				   GNOME_PILOT_CONDUIT (conduit),
 				   GnomePilotConduitSyncTypeMergeToPilot);
 	err = gnome_pilot_conduit_standard_merge_to_pilot (conduit, dbinfo);
-	orbed_notify_conduit_end (dbinfo->pilotInfo->name,
+	dbus_notify_conduit_end (dbinfo->pilotInfo->name,
 				   GNOME_PILOT_CONDUIT (conduit));
 	return err;
 }
@@ -478,13 +490,13 @@ conduit_merge_from_pilot (GnomePilotConduitStandard *conduit, GnomePilotDBInfo *
 	g_return_val_if_fail (conduit != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PILOT_CONDUIT_STANDARD (conduit), -1);
 
-	write_sync_type_line_to_pilot (dbinfo, GNOME_PILOT_CONDUIT (conduit), _("Merge from pilot %s\n"));
+	write_sync_type_line_to_pilot (dbinfo, GNOME_PILOT_CONDUIT (conduit), _("Merge from PDA %s\n"));
 
-	orbed_notify_conduit_start (dbinfo->pilotInfo->name,
+	dbus_notify_conduit_start (dbinfo->pilotInfo->name,
 				   GNOME_PILOT_CONDUIT (conduit),
 				   GnomePilotConduitSyncTypeMergeFromPilot);
 	err = gnome_pilot_conduit_standard_merge_from_pilot (conduit, dbinfo);
-	orbed_notify_conduit_end (dbinfo->pilotInfo->name,
+	dbus_notify_conduit_end (dbinfo->pilotInfo->name,
 				   GNOME_PILOT_CONDUIT (conduit));
 	return err;
 }
@@ -501,8 +513,8 @@ conduit_sync_default (GnomePilotConduitStandard *conduit, GnomePilotDBInfo *dbin
 	g_return_val_if_fail (dbinfo!=NULL, -1);
 	g_return_val_if_fail (dbinfo->pu!=NULL, -1);
 
-	conduit_config = gtk_object_get_data (GTK_OBJECT (conduit), GPCC_LABEL);
-	manager = gtk_object_get_data (GTK_OBJECT (conduit), GPCM_LABEL);
+	conduit_config = g_object_get_data (G_OBJECT (conduit), GPCC_LABEL);
+	manager = g_object_get_data (G_OBJECT (conduit), GPCM_LABEL);
 	g_return_val_if_fail (conduit_config!=NULL,-1);
 	if (conduit_config->first_sync_type != GnomePilotConduitSyncTypeNotSet) {
 		const char *conduit_name = gnome_pilot_conduit_management_get_name (manager);
@@ -550,7 +562,7 @@ install_db_foreach (GPilotRequest *req, GnomePilotInstallCarrier *carrier) {
 	set_callbacks (FALSE, carrier->conduit, pilot_name);
 
 	g_free (pilot_name);
-	orbed_notify_completion (&req);
+	dbus_notify_completion (&req);
 }
 
 static void
@@ -580,7 +592,7 @@ do_restore_foreach (GPilotRequest *req, GnomePilotInstallCarrier *carrier)
 
 	pilot_name = pilot_name_from_id (carrier->pu->userID, carrier->context);
 	/* FIXME: if carrier.conduit_list is NULL, the user hasn't
-	  enabled any file conduits. The corba restore call should
+	  enabled any file conduits. The dbus restore call should
 	  throw an exception or something.  */
 	iterator = carrier->conduit_list;
 	while ( iterator != NULL ) {
@@ -608,7 +620,7 @@ do_restore_foreach (GPilotRequest *req, GnomePilotInstallCarrier *carrier)
 	}
 
 	g_free (pilot_name);
-	orbed_notify_completion (&req);
+	dbus_notify_completion (&req);
 }
 
 static void
@@ -619,7 +631,7 @@ gpilot_unload_conduit_foreach (GnomePilotConduit *conduit,
 	g_return_if_fail (conduit != NULL);
 	g_return_if_fail (GNOME_IS_PILOT_CONDUIT (conduit));
 	
-	manager = gtk_object_get_data (GTK_OBJECT (conduit), GPCM_LABEL);
+	manager = g_object_get_data (G_OBJECT (conduit), GPCM_LABEL);
 	g_assert (manager!=NULL);
 	/* FIXME: _destroy_conduit doesn't g_module_close... is this an issue ? 
 	 note: the shlib loader in gnome_pilot_conduit_management now closes
@@ -636,13 +648,14 @@ gpilot_load_conduits (GPilotContext *context,
 		      GList **blist,
 		      GList **flist)
 {
-	int i, cnt;
-	gchar *prefix;
+	int i;
+	gsize cnt;
 	GnomePilotConduit *conduit;
 	guint32 pilotId;
 	gchar **conduit_name;
 	GnomePilotConduitManagement *manager;
 	GnomePilotConduitConfig *conduit_config;
+	GKeyFile *kfile;
 
 	pilotId = pilot->pilot_id;
 
@@ -651,13 +664,12 @@ gpilot_load_conduits (GPilotContext *context,
 	*flist = NULL;
 
 	/* Read the conduit configuration */
-	prefix = g_strdup_printf ("gnome-pilot.d/conduits%d/General/", pilot->pilot_id);
-	gnome_config_push_prefix (prefix);
-	gnome_config_get_vector ("conduits",&cnt,&conduit_name);
-	gnome_config_pop_prefix ();
-	g_free (prefix);
-	g_message (_("Instantiating %d conduits..."), cnt);
-	for (i = 0;i<cnt;i++) {
+	kfile = get_conduits_kfile (pilot->pilot_id);
+	conduit_name = g_key_file_get_string_list (kfile, "General", "conduits", &cnt, NULL);
+	g_key_file_free (kfile);
+
+	g_message (_("Instantiating %d conduits..."), (gint) cnt);
+	for (i = 0;i<cnt ;i++) {
 		gint err;
 		manager = gnome_pilot_conduit_management_new (conduit_name[i], GNOME_PILOT_CONDUIT_MGMT_ID);
 		if (manager == NULL) {
@@ -678,11 +690,11 @@ gpilot_load_conduits (GPilotContext *context,
 			continue;
 		}
 		
-		gtk_object_set (GTK_OBJECT (conduit),
+		g_object_set (G_OBJECT (conduit),
 				GNOME_PILOT_CONDUIT_PROP_PROGRESS_STEPPING (context->progress_stepping),
 				NULL);
-		gtk_object_set_data (GTK_OBJECT (conduit), GPCM_LABEL, manager);
-		gtk_object_set_data (GTK_OBJECT (conduit), GPCC_LABEL, conduit_config);
+		g_object_set_data (G_OBJECT (conduit), GPCM_LABEL, manager);
+		g_object_set_data (G_OBJECT (conduit), GPCC_LABEL, conduit_config);
 
 		if (GNOME_IS_PILOT_CONDUIT_BACKUP (conduit)) {
 			*blist = g_list_append (*blist, conduit);
@@ -701,6 +713,7 @@ gpilot_load_conduits (GPilotContext *context,
 	g_free (conduit_name);
 	g_message ("Instantiated %d backup conduits, %d file conduits, %d other conduits",
 		   g_list_length (*blist), g_list_length (*flist), g_list_length (*clist));
+
 }
 
 void
@@ -761,8 +774,8 @@ gpilot_start_unknown_restore (int pfd,
 	if (result) {
 		struct PilotUser pu;
 		char *tmp;
-		tmp = g_strdup_printf (_("Setting user/id to %d/%s..."), pilot->pilot_id, pilot->pilot_username);
-		orbed_notify_daemon_message (pilot->name, NULL, tmp);
+		tmp = g_strdup_printf (_("Setting id/owner to %d/%s..."), pilot->pilot_id, pilot->pilot_username);
+		dbus_notify_daemon_message (pilot->name, NULL, tmp);
 		g_snprintf (pu.username, 127, "%s", pilot->pilot_username);
 		pu.userID = pilot->pilot_id;
 		dlp_WriteUserInfo (pfd, &pu);
@@ -793,7 +806,7 @@ run_conduit_sync_foreach (GPilotRequest *req,
 	if (conduit==NULL) {
 		/* FIXME: remove the bloody request, or the user will get this
 		   message every time */
-		g_warning (_("Conduit %s requested, but not found"),
+		g_warning (_("Conduit %s requested, but not found.  Is it enabled?"),
 			   req->parameters.conduit.name);
 	} else {
 		if (GNOME_IS_PILOT_CONDUIT_BACKUP (conduit)) {
@@ -819,7 +832,7 @@ run_conduit_sync_foreach (GPilotRequest *req,
 					  carrier->context); 
 		}
 	}
-	orbed_notify_completion (&req);
+	dbus_notify_completion (&req);
 }
 
 gboolean gpilot_initial_synchronize_operations (int pilot_socket,
@@ -1038,10 +1051,10 @@ gpilot_add_log_entry (int pilot_socket,
 void 
 gpilot_manager_save_databases (GPilotPilot *pilot_info, 
 			      GSList *databases) {
-	gchar *pfx;
 	char **charlist;
 	int cnt=0;
 	GSList *ptr;
+	GKeyFile *kfile;
 
 	charlist = g_new0(char*, g_slist_length (databases)+1);
 
@@ -1050,17 +1063,15 @@ gpilot_manager_save_databases (GPilotPilot *pilot_info,
 		cnt++;
 	}
 
-	pfx = g_strdup_printf ("/gnome-pilot.d/PilotCache%d/Databases/",
-			      pilot_info->pilot_id);
-	gnome_config_push_prefix (pfx);
-	gnome_config_set_vector ("databases",
-				g_slist_length (databases),
-				(const char**)charlist);
-	gnome_config_pop_prefix ();
-	gnome_config_sync ();
+	kfile = get_pilot_cache_kfile (pilot_info->pilot_id);
+	g_key_file_set_string_list (kfile, "Databases", "databases",
+				(const char**)charlist,
+				g_slist_length (databases));
+	save_pilot_cache_kfile (kfile, pilot_info->pilot_id);
+	g_key_file_free (kfile);
+
 	/* FIXME: is this okay ? */
 	g_slist_foreach (databases,(GFunc)g_free, NULL);
-	g_free (pfx);
 	g_free (charlist);
 	g_slist_free (databases);
 }
