@@ -5,13 +5,11 @@
  */
 #include <config.h>
 
-#include <gnome.h>
-#include <bonobo-activation/bonobo-activation.h>
+#include <glib/gi18n.h>
 
-#include <gpilotd/gnome-pilot-client.h>
+#include <gnome-pilot-client.h>
 
 GnomePilotClient *gpc;
-CORBA_Environment ev;
 
 int     arg_pause=0,
 	arg_unpause=0,
@@ -19,7 +17,8 @@ int     arg_pause=0,
 	arg_restart=0,
 	arg_setuser = 0,
 	arg_listbases = 0,
-	arg_monitorall=0,
+	arg_listusers = 0,
+	arg_listcradles = 0,
 	arg_monitor,
 	arg_getinfo = 0;
 char
@@ -35,24 +34,25 @@ GList *outstanding_requests;
 
 gboolean stay_alive = TRUE;
 
-static const struct poptOption options[] = {
-	{"getinfo", '\0', POPT_ARG_NONE, &arg_getinfo, 0, N_("Get System Info"), NULL},
-	{"pause", 'p', POPT_ARG_NONE, &arg_pause, 0, N_("Pause daemon"), NULL},
-	{"unpause", 'u', POPT_ARG_NONE, &arg_unpause, 0, N_("Unpause daemon"), NULL},
-	{"restart", 'r', POPT_ARG_NONE, &arg_restart, 0, N_("Restart daemon"), NULL},
-	{"setuser", 'S', POPT_ARG_NONE, &arg_setuser, 0, N_("Set user id and name"), NULL},
-	{"pilotid" , '\0', POPT_ARG_INT, &arg_id, 0, N_("ID for the PDA"), N_("ID")},
-	{"install",'\0', POPT_ARG_STRING, &arg_install, 0 , N_("Install file"), N_("FILE")},
-	{"restore", '\0', POPT_ARG_STRING, &arg_restore, 0, N_("Restore directory"), N_("DIRECTORY")},
-	{"conduit", '\0', POPT_ARG_STRING, &arg_conduit, 0, N_("Run conduit"), N_("CONDUIT")},
-	{"userid", '\0', POPT_ARG_STRING, &arg_username, 0, N_("Username to set"), N_("USERNAME")},
-	{"monitor", 'm', POPT_ARG_NONE, &arg_monitor, 0, N_("Monitor the specified PDA's actions"), NULL},
-	{"monitorall", 'A', POPT_ARG_NONE, &arg_monitorall, 0, N_("Monitor all PDAs"), NULL},
-	{"listpilots", '\0', POPT_ARG_STRING, &arg_list_by_login, 0, N_("list PDAs by login (all for all pilots)"), N_("LOGIN")},
-	{"pilot", '\0', POPT_ARG_STRING, &arg_pilot, 0, N_("Specify PDA to operate on (defaults to MyPDA)"), N_("PILOTNAME")},
-	{"cradle", '\0', POPT_ARG_STRING, &arg_cradle, 0, N_("Specify a cradle to operate on (defaults to Cradle0)"), N_("CRADLENAME")},
-	{"listbases", 'l', POPT_ARG_NONE, &arg_listbases, 0, N_("List the specified PDA's bases"), NULL},
-	{NULL, '\0', 0, NULL, 0} /* end the list */
+static GOptionEntry options[] = {
+	{"getinfo", '\0', 0, G_OPTION_ARG_NONE, &arg_getinfo, N_("Get System Info"), NULL},
+	{"pause", 'p', 0, G_OPTION_ARG_NONE, &arg_pause, N_("Pause daemon"), NULL},
+	{"unpause", 'u', 0, G_OPTION_ARG_NONE, &arg_unpause, N_("Unpause daemon"), NULL},
+	{"restart", 'r', 0, G_OPTION_ARG_NONE, &arg_restart, N_("Restart daemon"), NULL},
+	{"setuser", 'S', 0, G_OPTION_ARG_NONE, &arg_setuser, N_("Set user id and name"), NULL},
+	{"pilotid" , '\0', 0, G_OPTION_ARG_INT, &arg_id, N_("ID for the PDA"), N_("ID")},
+	{"install",'\0', 0, G_OPTION_ARG_STRING, &arg_install, N_("Install file"), N_("FILE")},
+	{"restore", '\0', 0, G_OPTION_ARG_STRING, &arg_restore, N_("Restore directory"), N_("DIRECTORY")},
+	{"conduit", '\0', 0, G_OPTION_ARG_STRING, &arg_conduit, N_("Run conduit"), N_("CONDUIT")},
+	{"username", '\0', 0, G_OPTION_ARG_STRING, &arg_username, N_("Username to set"), N_("USERNAME")},
+	{"monitor", 'm', 0, G_OPTION_ARG_NONE, &arg_monitor, N_("Monitor all PDAs"), NULL},
+	{"listpilots", '\0', 0, G_OPTION_ARG_STRING, &arg_list_by_login, N_("list PDAs by login (all for all pilots)"), N_("LOGIN")},
+	{"pilot", '\0', 0, G_OPTION_ARG_STRING, &arg_pilot, N_("Specify PDA to operate on (defaults to MyPDA)"), N_("PILOTNAME")},
+	{"cradle", '\0', 0, G_OPTION_ARG_STRING, &arg_cradle, N_("Specify a cradle to operate on (defaults to Cradle0)"), N_("CRADLENAME")},
+	{"listusers", '\0', 0, G_OPTION_ARG_NONE, &arg_listusers, N_("List users"), NULL},
+	{"listcradles", '\0', 0, G_OPTION_ARG_NONE, &arg_listcradles, N_("List cradles"), NULL},
+	{"listbases", 'l', 0, G_OPTION_ARG_NONE, &arg_listbases, N_("List the specified PDA's bases"), NULL},
+	{NULL},
 };
 
 static void
@@ -72,6 +72,87 @@ pilot_disconnect (GnomePilotClient *gpc,
 	(*b) = FALSE;
 };
 
+static void
+daemon_pause (GnomePilotClient *gpc, 
+              gboolean          on_ff, 
+              gpointer          data) {
+	g_message ("daemon pause: %d", on_ff);
+}
+
+static void
+conduit_message (GnomePilotClient *gpc, 
+                 const gchar      *pilot_id,
+                 const gchar      *conduit,
+                 const gchar      *message,
+                 gpointer          data) {
+        g_message ("conduit message: pilot=%s, conduit=%s", pilot_id, conduit);
+        g_message ("conduit message: %s", message);
+}
+
+static void
+daemon_message (GnomePilotClient *gpc, 
+                 const gchar      *pilot_id,
+                 const gchar      *conduit,
+                 const gchar      *message,
+                 gpointer          data) {
+        g_message ("daemon message: pilot=%s, conduit=%s", pilot_id, conduit);
+        g_message ("daemon message: %s", message);
+}
+
+static void
+conduit_error (GnomePilotClient *gpc, 
+                 const gchar      *pilot_id,
+                 const gchar      *conduit,
+                 const gchar      *message,
+                 gpointer          data) {
+        g_message ("conduit error: pilot=%s, conduit=%s", pilot_id, conduit);
+        g_message ("conduit error: %s", message);
+}
+
+static void
+daemon_error (GnomePilotClient *gpc, 
+                 const gchar      *pilot_id,
+                 const gchar      *message,
+                 gpointer          data) {
+        g_message ("daemon error: pilot=%s", pilot_id);
+        g_message ("daemon error: %s", message);
+}
+static void
+start_conduit (GnomePilotClient *gpc, 
+               const gchar      *pilot_id, 
+               const gchar      *conduit, 
+               const gchar      *database,
+	       gpointer          data) {
+	g_message ("start_conduit:   pilot_id=%s, conduit=%s, database=%s", pilot_id, conduit, database);
+};
+
+static void
+end_conduit (GnomePilotClient *gpc, 
+             const gchar      *pilot_id, 
+             const gchar      *conduit, 
+	     gpointer          data) {
+	g_message ("end_conduit:   pilot_id=%s, conduit=%s", pilot_id, conduit);
+};
+
+static void
+progress_conduit (GnomePilotClient *gpc, 
+                  const gchar      *pilot_id, 
+                  const gchar      *conduit, 
+                  guint32           current,
+                  guint32           total,
+                  gpointer          data) {
+	g_message ("progress_conduit:   pilot_id=%s, conduit=%s, current=%u, total=%u", pilot_id, conduit, current, total);
+};
+
+static void
+progress_overall (GnomePilotClient *gpc, 
+                  const gchar      *pilot_id, 
+                  guint32           current,
+                  guint32           total,
+                  gpointer          data) {
+	g_message ("progress_overall:   pilot_id=%s, current=%u, total=%u", pilot_id, current, total);
+};
+
 static void
 pilot_request_completed (GnomePilotClient *gpc, gchar *pilot_id, gint handle, gpointer data) {
 	GList *iterator;
@@ -105,7 +186,7 @@ user_info (GnomePilotClient *gpc,
 {
 	g_message ("Pilot in cradle %s :", cradle);
 	g_message ("User name %s", userinfo->username);
-	g_message ("User id   %d", userinfo->userID);
+	g_message ("User id   %lu", userinfo->userID);
 }
 	     
 
@@ -115,21 +196,45 @@ system_info (GnomePilotClient *gpc,
 	     const GNOME_Pilot_SysInfo *sysinfo)
 {
 	g_message ("Pilot in cradle %s :", cradle);
-	g_message ("%dKB ROM", sysinfo->romSize);
-	g_message ("%dKB RAM", sysinfo->ramSize);
-	g_message ("%dKB RAM free", sysinfo->ramFree);
+	g_message ("%ldKB ROM", sysinfo->romSize);
+	g_message ("%ldKB RAM", sysinfo->ramSize);
+	g_message ("%ldKB RAM free", sysinfo->ramFree);
 	g_message ("Systemboard is ``%s''", sysinfo->name);
 	g_message ("Manufacturer is ``%s''", sysinfo->manufacturer);
-	g_message ("Creation/Version is %d/%d", sysinfo->creation, sysinfo->romVersion);
+	g_message ("Creation/Version is %ld/%ld", sysinfo->creation, sysinfo->romVersion);
 }
 
 static void 
 the_loop (void) {
 	while (stay_alive) {
-		g_main_iteration (TRUE);
+		g_main_context_iteration (NULL, TRUE);
 	}
 }
 
+static void list_users (void) {
+	GList *list = NULL,*ptr;
+
+	gnome_pilot_client_get_users (gpc, &list);
+	if (list)
+		for (ptr = list; ptr; ptr = ptr->next) {
+			g_message ("user %s", (char*)ptr->data);
+		}
+	else 
+		g_message ("No users");
+}
+
+static void list_cradles (void) {
+	GList *list = NULL,*ptr;
+
+	gnome_pilot_client_get_cradles (gpc, &list);
+	if (list)
+		for (ptr = list; ptr; ptr = ptr->next) {
+			g_message ("cradle %s", (char*)ptr->data);
+		}
+	else 
+		g_message ("No cradle");
+}
+
 static void list_bases (void) {
 	GList *list = NULL,*ptr;
 
@@ -226,7 +331,7 @@ revive (char *rest)
 
 	g_message ("Reviving %d to %s from %s",arg_id,arg_cradle,arg_restore);
 	user.userID = arg_id;
-	user.username = CORBA_string_dup (arg_username);
+	user.username = g_strdup (arg_username);
 	err = gnome_pilot_client_set_user_info (gpc,
 					       arg_cradle,
 					       user,
@@ -287,7 +392,7 @@ set_user (void)
 		g_warning ("Please set a pilotid using --pilotid");
 		abort = TRUE;
 	}	if (arg_username==NULL) {
-		g_warning ("Please set a username using --userid");
+		g_warning ("Please set a username using --username");
 		abort = TRUE;
 	}
 	if (arg_cradle==NULL) {
@@ -298,7 +403,7 @@ set_user (void)
 	if (!abort) {
 		g_message ("Setting userid/name %d/%s on %s",arg_id,arg_username,arg_cradle);
 		user.userID = arg_id;
-		user.username = CORBA_string_dup (arg_username);
+		user.username = g_strdup (arg_username);
 		err = gnome_pilot_client_set_user_info (gpc,
 						       arg_cradle,
 						       user,
@@ -342,8 +447,9 @@ get_system_info (void)
 
 int
 main (int argc, char *argv[]) {
-        poptContext pctx;
-	
+	GOptionContext *option_context;
+	GError *error = NULL;
+
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
 	textdomain (PACKAGE);
@@ -353,8 +459,15 @@ main (int argc, char *argv[]) {
 		exit (1);
 	}
 
-	gnome_init_with_popt_table ("gpilotd-client","1.0",
-				    argc, argv, options, 0, &pctx);
+	option_context = g_option_context_new (PACKAGE);
+	g_option_context_add_main_entries (option_context, options, NULL);
+
+	if (!g_option_context_parse(option_context, &argc, &argv, &error)) {
+		g_error (_("Error parsing commandline arguments: %s"), error->message);
+		exit (1);
+	}
+
+	gtk_init (&argc, &argv);
 
 	g_message (_("\nBEWARE!!\nThis is a tool for certain parts of the gnome-pilot package.\nUnless you know what you're doing, don't use this tool."));
 
@@ -383,37 +496,37 @@ main (int argc, char *argv[]) {
 		revive (NULL);
 	} else if (arg_restore!=NULL) {
 		restore (arg_restore);
-	} else if (arg_conduit!=NULL) {
+	} else if (arg_conduit!=NULL && !arg_monitor) {
 		conduit (arg_conduit);
 	} else if (arg_setuser) {
 		set_user ();
+	} else if (arg_listusers) {
+		list_users ();
+	} else if (arg_listcradles) {
+		list_cradles ();
 	} else if (arg_listbases) {
 		list_bases ();
 	} else if (arg_list_by_login) {
 		list_by_login ();
 	} else if (arg_monitor) {
 		g_message ("Monitor on %s",arg_pilot);
-		gnome_pilot_client_monitor_on (gpc,arg_pilot);
-		gnome_pilot_client_notify_on (gpc,GNOME_Pilot_NOTIFY_CONNECT);
-		gnome_pilot_client_notify_on (gpc,GNOME_Pilot_NOTIFY_DISCONNECT);
 		
 		gnome_pilot_client_connect__pilot_connect (gpc, pilot_connect, NULL);
 		gnome_pilot_client_connect__pilot_disconnect (gpc, pilot_disconnect, &stay_alive);
+		gnome_pilot_client_connect__daemon_pause (gpc, daemon_pause, NULL);
+		gnome_pilot_client_connect__message_conduit (gpc, conduit_message, NULL);
+		gnome_pilot_client_connect__message_daemon (gpc, daemon_message, NULL);
+		gnome_pilot_client_connect__error_conduit (gpc, conduit_error, NULL);
+		gnome_pilot_client_connect__error_daemon (gpc, daemon_error, NULL);
 
+                if (arg_conduit) {
+                        gnome_pilot_client_connect__start_conduit (gpc, start_conduit, NULL);
+                        gnome_pilot_client_connect__end_conduit (gpc, end_conduit, NULL);
+                        gnome_pilot_client_connect__progress_conduit (gpc, progress_conduit, NULL);
+                        gnome_pilot_client_connect__progress_overall (gpc, progress_overall, NULL);
+                }
 		the_loop ();
 
-		gnome_pilot_client_notify_off (gpc,GNOME_Pilot_NOTIFY_CONNECT);
-		gnome_pilot_client_notify_off (gpc,GNOME_Pilot_NOTIFY_DISCONNECT);
-		gnome_pilot_client_monitor_off (gpc,arg_pilot);
-	} else if (arg_monitorall) {
-		g_message ("Monitor on all");
-		gnome_pilot_client_monitor_on_all_pilots (gpc);
-
-		gnome_pilot_client_connect__pilot_connect (gpc, pilot_connect, NULL);
-		gnome_pilot_client_connect__pilot_disconnect (gpc, pilot_disconnect, &stay_alive);
-
-		the_loop ();
-		gnome_pilot_client_monitor_off_all_pilots (gpc);		
 	} else if (arg_getinfo) {
 		g_message ("Get SystemInfo");
 		get_system_info ();
